package document

import (
	"fmt"
	"log"
	"os"
)

// File contains the raw data of a file coming from a scanner.
type File struct {
	Name string
	Data []byte
}

// LoadFile reads a file and returns it as a File.
func LoadFile(filename string) (File, error) {
	data, err := os.ReadFile(filename)
	if err != nil {
		return File{}, fmt.Errorf("couldn't read file: %w", err)
	}

	return File{
		Name: filename,
		Data: data,
	}, nil
}

// LoadFile reads a file and returns it as a File.
// This will panic on any error.
func MustLoadFile(filename string) File {
	file, err := LoadFile(filename)
	if err != nil {
		log.Panicf("Failed to load file: %v.", err)
	}

	return file
}
